#include "BooleanExpression.h"

namespace Inter4ql {

	void BooleanExpression::add_element(Element_Type type, data_type data)
	{
		elems.push_back((bool_elem){type, data});
	}

	int BooleanExpression::calculate(int i, int t, int f, int u)
	{
		//std::cout << "Calculate with " << i << " " << t << " " << f << " " << u << std::endl;
		std::stack<int> data_stack;
		for (const bool_elem& element : elems)
		{
			switch(element.type){
				case NUMBER: data_stack.push(element.data.number); break;
				case T: data_stack.push(t); break;
				case F: data_stack.push(f);break;
				case I: data_stack.push(i); break;
				case U: data_stack.push(u); break;
				case ALL: data_stack.push(t+f+i+u); break;
				default:
				{
					int b = data_stack.top(); data_stack.pop();
					int a = data_stack.top(); data_stack.pop();
					if (element.type == OP)
					{
						switch(element.data.operation) {
						case AND: data_stack.push( (a <= 0 || b <= 0) ? 0 : 1); break;
						case OR: data_stack.push( (a <= 0 && b <= 0) ? 0 : 1); break;
						case PLUS: data_stack.push(a+b); break;
						case MINUS: data_stack.push(a-b); break;
						case MUL: data_stack.push(a*b); break;
						case DIV: data_stack.push(b == 0 ? 0 : a/b); break;
						}
					} else if (element.type == COMP) {
						switch(element.data.comparison) {
						case EQ: data_stack.push(a == b); break;
						case NEQ: data_stack.push(a != b); break;
						case GT: data_stack.push(a > b); break;
						case GTE: data_stack.push(a >= b); break;
						case LT: data_stack.push(a < b); break;
						case LTE: data_stack.push(a <= b); break;
						}
					}
				}
			}
		}
		return data_stack.top();
	}

	void BooleanExpression::add_boolean_expression(BooleanExpression b)
	{
		elems.insert(elems.end(), b.elems.begin(), b.elems.end());
	}

}

