#ifndef __DISPOSABLE_H__
#define __DISPOSABLE_H__

#include <vector>
#include <map>

namespace Inter4ql {
	/** @brief Enumeration type for types of objects that can be stored in Disposable object
	*/
	struct disp_type {
		enum type {
			PROGRAM,
			MODULE_LOCAL_VECTOR,
			MODULE_LOCAL,
			MODULE_GROUP,
			MEMBERS_VECTOR,
			DOMAIN_VECTOR,
			DOMAIN_suf,
			RELATION_VECTOR,
			RELATION,
			RULE_VECTOR,
			RULE,
			FACT_VECTOR,
			FACT,
			FACT_VECTOR_VECTOR,
			VALUE_VECTOR,
			VALUE,
			MODULE_BELIEF,
			MODULE_ACTION,
			MODULE_PROBLEM,
			CONSTRAINTS,
			CONSTRAINT_ENTRY,
			QUANTIFIER_ENTRY,
			QUANTIFIERS_VECTOR,
			CONSTRAINT_ENTRY_VECTOR,
			EXPRESSION_VECTOR,
			EXPRESSION,
		};
	};

	/** @brief structure that handles memory managment - adds objecst and frees them
	*/
	class Disposable {
		public:
			/** @brief constructor
			 *
			 */
			Disposable();
			/** @brief destructor
			 *
			 */
			~Disposable();
			/** @brief adds object to object list
			 *
			 * @param address pointer object to be added
			 * @param object_type type of object that should be added
			 */
			void add(void* address, disp_type::type object_type);
			/** @brief removes object from object list
			 *
			 * @param address pointer object to be removed
			 * @param object_type type of object that should be removed
			 */
			void remove(void* address, disp_type::type object_type);
			/** @brief clears object list and frees all objects
			 *
			 */
			void clear();
			/** @brief if true, destructors in objects will not free their internals
			 *
			 */
			static bool errorMode;
		private:
			std::map<disp_type::type, std::vector<void*>*>* objects;
			bool debug;
	};
}

#endif /* __DISPOSABLE_H__ */

