#ifndef __DOMAIN_H__
#define __DOMAIN_H__

#include <string>
#include "Types.h"

namespace Inter4ql {

	/** @brief Domain class - class that points to a data type
	 *
	 */
	class Domain {
	public:
		/** @brief constructor that creates a connection between name and type
		 *
		 * @param a_name name of a domain
		 * @param type type of a domain
		 */
		explicit Domain(std::string a_name, variable_type::type type = variable_type::INVALID);
		/** @brief constructor that creates a default named type
		 *
		 * @param a_type type of a domain
		 */
		explicit Domain(Inter4ql::variable_type::type a_type);
		/** @brief destructor
		 *
		 */
		~Domain();
		/** @brief getter for name of a domain
		 *
		 * @return name of the domain
		 */
		std::string get_name() const;
		/** @brief getter for type of a domain
		 *
		 * @return type of the domain
		 */
		variable_type::type get_type() const;
		/** @brief setter for type of a domain
		 *
		 * @param _t type of a domain
		 */
		void set_type(variable_type::type _t);
		/** @brief returns string with domain information
		 *
		 * @return domain information packed in a std::string
		 */
		std::string print();
	private:
		std::string name;
		Inter4ql::variable_type::type type;
	};

	bool operator==(const Domain& a, const Domain& b);
	bool operator!=(const Domain& a, const Domain& b);
}
#endif /* DOMAIN_H */

