#ifndef __MODULE_ACTION_H__
#define __MODULE_ACTION_H__

#include <vector>
#include <string>

#include "Types.h"
#include "Functions.h"
#include "Rule.h"
#include "ModuleGeneric.h"

namespace Inter4ql {

	class Result;
	class Rule;
	class Fact;
	class Value;

	/** @brief Implementation of ModuleAction for modules with reasoning
	 */
	class ModuleAction : public ModuleGeneric {
	public:
		/** @brief constructor
		 *
		 * @param _name name of a module
		 * @param _constraints initial list of constraints
		 * @param _members list of members
		 */
		ModuleAction(std::string _name,
			std::vector<Value*>* _args,
			Expression* _preconditions,
			std::vector<Rule*>* _add,
			std::vector<Rule*>* _remove,
			Expression* composite_expr);
		/** @brief destructor
		 *
		 */
		~ModuleAction();

		std::string get_name() const;
		std::string print();
		module_type::type get_module_type() const;

		std::vector<Value*>* get_args();
		Expression* get_preconditions();
		Expression* get_composite_expr();
		std::vector<Rule*>* get_add_rules();
		std::vector<Rule*>* get_remove_rules();

		bool is_composite();

		void reason();
		Result ask_for_result(Fact * term) const;

		/** @brief setter for modules needed in this module
		 *
		 * @param _modules list of modules
		 */
		void set_modules(std::vector<ModuleGeneric *> *_modules) override;

	private:
		std::vector<Value*>* args;
		Expression* preconditions;
		std::vector<Rule*>* add;
		std::vector<Rule*>* remove;
		std::string name;
		std::vector<ModuleGeneric *> *modules;
		Expression* composite_expr;

		std::vector<ModuleGeneric *> *runnable_module_list;
	};
}

#endif  /* __MODULE_ACTION_H__ */

