#ifndef __MODULEGENERIC_CC__
#define __MODULEGENERIC_CC__

#include "ModuleGeneric.h"

#include "Application.h"
#include "Domain.h"
#include "Exception.h"
#include "Functions.h"
#include "Expression.h"
#include "Logic.h"
#include "Result.h"

namespace Inter4ql {
	Result ModuleGeneric::evaluate_expression_bf(Expression* expr) const
	{
		return evaluate_expression(expr);
	}

	Result ModuleGeneric::evaluate_expression(Expression* expr) const
	{
		bool debug_evaluation = Application::app_instance->debug_evaluation;

		switch(expr->get_type())
		{
			case expression_type::AND: {
						Result res = evaluate_expression(expr->get_left_expression());
						Result res2 = evaluate_expression(expr->get_right_expression());
						res.sum_Results_OP(res2, operation_type::AND);
						if (debug_evaluation)
							printf("Returning from evaluate_expression - and! %s\n", res.print(true, true).c_str());
						return res;
					  }
				break;
			case expression_type::IMPLIES: {
						Result res = evaluate_expression(expr->get_left_expression());
						Result res2 = evaluate_expression(expr->get_right_expression());
						res.sum_Results_OP(res2, operation_type::IMPLICATION);
						if (debug_evaluation)
							printf("Returning from evaluate_expression - impl! %s\n", res.print(true, true).c_str());
						return res;
					  }
				break;
			case expression_type::OR: {
						Result res = evaluate_expression(expr->get_left_expression());
						Result res2 = evaluate_expression(expr->get_right_expression());
						res.sum_Results_OP(res2, operation_type::OR);
						if (debug_evaluation)
							printf("Returning from evaluate_expression - or! %s\n", res.print(true, true).c_str());
						return res;
					  }
				break;
			case expression_type::NOT: {
						Result res = evaluate_expression(expr->get_left_expression());
						res.negate();
						if (debug_evaluation)
							printf("Returning from evaluate_expression - neg! %s\n", res.print(true, true).c_str());
						return res;
					  }
				break;
			case expression_type::QUANTIFIED: {
						if (!this->get_domains())
							throw new Exception("Quantified expression tried to be executed on non-domain module!\n");

						std::vector<ModuleGeneric*>* global_modules = Application::app_instance->public_get_modules();

						int rating; // not used
						Result res = evaluate_expression_global(expr, this, global_modules, this->get_domains(), rating);
						if (debug_evaluation)
							printf("Returning from evaluate_expression - quantified! %s\n", res.print(true, true).c_str());
						return res;
					  }
				break;
			case expression_type::TERM: {
						Fact* term = expr->get_term();
						if (debug_evaluation)
							printf("Evaluating term (evaluate_expression) %s\n", term->print(true).c_str());
						Result term_result = ask_for_result(expr->get_term());

						std::map<logic_type::type, bool> in;
						if (expr->get_term()->is_set_query())
						{
							std::string s = expr->get_term()->get_set_string();
							in.insert(std::make_pair(logic_type::UNKNOWN      , s[0] == '1'));
							in.insert(std::make_pair(logic_type::FALSE        , s[1] == '1'));
							in.insert(std::make_pair(logic_type::TRUE         , s[2] == '1'));
							in.insert(std::make_pair(logic_type::INCONSISTENT , s[3] == '1'));

							for (auto& t : term_result.get_valuation_destructive())
							{
								if (in.find(t.second)->second)
									t.second = logic_type::TRUE;
								else
									t.second = logic_type::FALSE;
							}
						}

						return term_result;
					  }
				break;
			default:
				return Result();
		}
		return Result();
	}

	std::vector<std::string> ModuleGeneric::get_references(Fact * term) {
		return std::vector<std::string>();
	}

	bool ModuleGeneric::contains_domain(Domain* d) const {
		std::vector<Domain*>* domains = get_domains();
		if (!domains)
			return false;
		for (std::vector<Domain*>::iterator it = domains->begin(); it != domains->end(); it++) {
			Domain* current = *it;
			if(current->get_name() == d->get_name() && current->get_type() == d->get_type())
				return true;
		}
		return false;
	}

	bool ModuleGeneric::contains_relation(Relation* r) const {
		std::vector<Relation*>* relations = get_relations();
		if (!relations)
			return false;
		for (std::vector<Relation*>::iterator it = relations->begin(); it != relations->end(); it++) {
			Relation* current = *it;
			//printf("Compare relation %s with %s\n", current->print().c_str(), r->print().c_str());
			if(current->get_name() == r->get_name()) {
				if (current->get_params()->size() == r->get_params()->size()) {
					bool params_match = true;
					for(int i = 0; i < current->get_params()->size(); i++) {
						if (*current->get_params()->at(i) != *r->get_params()->at(i))
							params_match = false;
					}
					if (params_match)
						return true;
				}
			}
		}
		return false;
	}
}

#endif
