#ifndef __MODULEGROUP_H__
#define __MODULEGROUP_H__

#include <vector>
#include <string>

#include "ModuleGeneric.h"
#include "Types.h"

namespace Inter4ql {

	class Rule;
	class Domain;
	class Result;
	class Relation;
	class Fact;
	class BooleanExpression;
	/** @brief Implementation of ModuleGeneric for modules with reasoning
	 */
	class ModuleGroup : public ModuleGeneric {
	public:
		/** @brief constructor
		 *
		 * @param a_name name of a module
		 * @param _domains initial vector of domains
		 * @param _relations initial vector of relations
		 * @param _rules initial vector of rules
		 * @param _facts initial vector of facts
		 */
		ModuleGroup(std::string a_name,
			std::vector<Domain*> *_domains,
			std::vector<Relation*> *_relations,
			std::vector<std::string> *_members);
		/** @brief destructor
		 *
		 */
		~ModuleGroup();
		Relation *get_relation(std::string s) const;
		virtual std::vector<Domain*> *get_domains() const;
		std::vector<Relation*> *get_relations() const;
		std::vector<std::string> *get_members() const;
		std::string get_name() const;
		std::string print();
		module_type::type get_module_type() const;
		variable_type::type get_domain_type(std::string name);

		void add_domain(Domain *d);
		void add_relation(Relation *r);

		void reason();
		Result ask_for_result(Fact * term) const;
		Result evaluate_expression(Expression * expr) const;
		/** @brief prepare formulas for merging group data
		 *
		 * @param t boolan expression for when value is true
		 * @param f boolan expression for when value is false
		 * @param t boolan expression for when value is inconsistent
		 */
		void add_whens(BooleanExpression * t, BooleanExpression * f, BooleanExpression * i);
	private:
		Result ask_for_result_generic(Fact * term, Expression * expr) const;

		std::vector<Domain *> *domains;
		std::vector<Domain *> *domainsToDelete;

		std::vector<Relation *> *relations;
		std::vector<std::string> *members;
		std::string name;
		BooleanExpression * true_when;
		BooleanExpression * false_when;
		BooleanExpression * inconsistent_when;
	};
}

#endif  /* __MODULEGROUP_H__ */

