#include <sstream>

#include "ModuleInternal.h"

#include "modules/Math.h"
#include "VariableSpace.h"
#include "Relation.h"
#include "Exception.h"
#include "Rule.h"

namespace Inter4ql {

	ModuleInternal::ModuleInternal(VariableSpace *_v) {
		this->variable_space = _v;
	}

	ModuleInternal::~ModuleInternal() {
		for(std::vector<Relation*>::iterator it = this->relations->begin(); it != this->relations->end(); it++) {
			delete (*it);
		}
		delete this->relations;
	}

	module_type::type ModuleInternal::get_module_type() const {
		return module_type::INTERNAL;
	}

	Relation *ModuleInternal::get_relation(std::string s) const {
		for (int i = 0; i < this->relations->size(); i++)
			if (this->relations->at(i)->get_name() == s)
				return this->relations->at(i);
		throw new Exception(std::string("There is no relation ") + s + " in module " + this->name +  "!\n");
	}

	std::vector<Relation*> *ModuleInternal::get_relations() const {
		return this->relations;
	}

	std::string ModuleInternal::get_name() const {
		return this->name;
	}

	std::string ModuleInternal::print() {
		std::stringstream ss;
		ss << "module " << this->name << ":\n";
		if (this->relations->size())
			ss << "relations:\n";
		for (int i = 0; i < this->relations->size(); i++)
			ss << "\t" << this->relations->at(i)->print() << "\n";
		ss << "end.\n";

		return ss.str();
	}

}
