#ifndef __PARSER_CC__
#define __PARSER_CC__

#include "Program.h"
#include "Disposable.h"
#include "Parser.h"
#include "Functions.h"
#include "Scanner.h"
#include "Exception.h"
#include "Output.h"

namespace Inter4ql {


	Parser::Parser(Inter4ql::Output *_o, std::istream* arg_yyin, std::ostream* arg_yyout) {
		this->line.program = 0;
		this->line.rule = 0;
		this->output = _o;
		this->scanner = 0;
		this->parser = 0;
		this->init(arg_yyin, arg_yyout);
	}

	/** @brief destructor
	 */
	Parser::~Parser() {
		if(this->scanner) delete this->scanner;
		if(this->parser) delete this->parser;
	}

	void Parser::init(std::istream* arg_yyin, std::ostream* aarg_yyout) {

		if (this->scanner) delete this->scanner;
		if (this->parser) delete this->parser;
		this->scanner = new Inter4ql::Scanner(arg_yyin, aarg_yyout);
		this->parser = new Inter4ql::LineParser(*this->scanner, this->line);

	}

	Inter4ql::parser_line *Parser::parse() {
		try {
			if (parser->parse() == 0) { /* gramatical analysis went well */

				//* only if we are in debug mode */
				//Inter4ql::DebugProgramTree::print_program(&tree);
				//* now we need to check if syntax is all good */
				return new Inter4ql::parser_line(this->line);

			} else { /* something went wrong */
				/* verbose error message was printed by parser.parse() method */
				/* we should never be here, exception was thrown and caught below */
				return 0;
			}

		} catch (Exception *s) {
			Disposable* toDelete = s->getDisp();
			if(toDelete) {
				toDelete->clear();
				delete toDelete;
			}

			this->output->print(s);
			delete s;
			return 0;
		}
	}

}

#endif
