#ifndef __RELATION_H__
#define __RELATION_H__

#include <string>
#include <vector>

namespace Inter4ql {

	class Domain;

	/** @brief class that handles relation instance inside modules
	 */
	class Relation {
	public:
		/** @brief constructor for Relation class
		 *
		 * @param a_name name of relation
		 * @param a_params list of types of parameters
		 * @param _module_name name of a module relation is inside
		 */
		Relation(std::string a_name, std::vector<Domain *> *a_params, std::string _module_name);
		/** @brief destructor for Relation class
		 *
		 */
		~Relation();
		/** @brief getter for relation params
		 *
		 * @return vector of relation parameters types
		 */
		std::vector<Domain*> *get_params();
		/** @brief getter for relation name
		 *
		 * @return relation name
		 */
		std::string get_name();
		/** @brief getter for module name relation is in
		 *
		 * @return module name
		 */
		std::string get_module();
		/** @brief setter for module name relation is in
		 *
		 * @param _m module name
		 */
		void set_module(std::string _m);
		/** @brief getter for value saing, whether this relation is queried for unknown facts
		 *
		 * @return isUnknown value
		 */
		bool is_unknown();
		/** @brief setter for value saing, whether this relation is queried for unknown facts
		 *
		 * @param value new isUnknown value
		 */
		void set_unknown(bool value);
		/** @brief returns string with relation information
		 *
		 * @return relation information packed in a std::string
		 */
		std::string print();
	private:
		bool isUnknown;
		std::string print_params();
		std::string name;
		std::vector<Domain *> *params;
		std::string module;
	};

}

#endif  /* __RELATION_H__ */

