#ifndef __RESULT_H__
#define __RESULT_H__

#include "Types.h"
#include "Valuation.h"

#include <utility>
#include <map>
#include <string>
#include <memory>
#include <vector>

namespace Inter4ql {

	class Value;
	class Result {
	public:
		Result();
		explicit Result(std::vector<std::string> names);
		explicit Result(std::vector<std::string> names, Valuation valuation);
		~Result();

		const std::vector<std::string>& get_variable_names() const;
		const Valuation& get_valuation() const;
		Valuation& get_valuation_destructive();

		/** @brief adds element to result
		 *
		 * @param vect values that are being valuated
		 * @param value logic value
		 * @returns true if adds successfully
		 */
		bool add_element(const std::vector<Value*>& vect, logic_type::type value);
		/** @brief adds variable name to result
		 *
		 * @param name name to be added
		 */
		void add_variable_name(const std::string name);
		/** @brief merges another Result into current one
		 *
		 * @param other another result
		 * @param operatio logical operation used for merging
		 */
		void sum_Results_OP(Result& other, operation_type::type operation);
		/** @brief negates all logic values
		 *
		 */
		void negate();
		/** @brief restricts result to only some variables
		 *
		 * @param names variables to preserve
		 */
		Result* trim_variables(std::vector<std::string> names);
		/** @brief applies operation to all logic values in valuation
		 *
		 * @param operation operation to be applied
		 * @param value logic values used as operand to operation
		 */
		void apply_operation(operation_type::type operation, logic_type::type value);
		/** @brief prints Result
		 *
		 * @param type prints types of element
		 * @param unknown prints unknown values
		 */
		std::string print(bool type = false, bool unknown = false);
	private:

		std::vector<std::string> variable_names;
		Valuation valuation;
		static Value dummy_plug;
	};
}

#endif  /* __RESULT_H__ */

