#include <string>
#include "Rule.h"
#include "Fact.h"
#include "Disposable.h"

namespace Inter4ql {

	Rule::Rule(Fact *_head, std::vector<std::vector<Fact*>*> *_body) {
		this->head = _head;
		this->body = _body;
	}

	Rule::~Rule() {
		if(!Disposable::errorMode) {
				for(std::vector<Value*>::iterator it = this->head->get_params()->begin(); it != this->head->get_params()->end(); it++) {
						if((*it)->get_type() == Inter4ql::variable_type::VARIABLE) delete (*it);
				}
				delete this->head;

				for(std::vector<std::vector<Fact*>*>::iterator it = this->body->begin(); it != this->body->end(); it++) {
						for(std::vector<Fact*>::iterator it1 = (*it)->begin(); it1 != (*it)->end(); it1++) {
								for(std::vector<Value*>::iterator it2 = (*it1)->get_params()->begin(); it2 != (*it1)->get_params()->end(); it2++) {
										if((*it2)->get_type() == Inter4ql::variable_type::VARIABLE) delete (*it2);
								}
								delete (*it1);
						}
						delete (*it);
				}
				delete this->body;
		} else {
				//std::cout << "DELETE RULE*" << std::endl;
		}
	}

	Rule::Rule(Rule* other) {
		this->head = new Fact(other->head);
		this->body = new std::vector<std::vector<Fact*>*>();
		for (auto section : *other->body) {
			auto group = new std::vector<Fact*>();
			for (auto val : *section) {
				group->push_back(new Fact(val));
			}
			this->body->push_back(new std::vector<Fact*>());
		}
	}

	Rule::Rule(Fact *_head, std::vector<Fact*> *_body) {
		this->head = _head;
		this->body = new std::vector<std::vector<Fact*>*>();
		this->body->push_back(_body);
	}

	Fact *Rule::get_head() {
		return this->head;
	}

	std::vector<std::vector<Fact*>*> *Rule::get_body(){
		return this->body;
	}

	std::string Rule::print() {
		std::string s = "";
		s += this->head->print() + " :- ";
		for (int i = 0; i < this->body->size(); i++){
			for (int j = 0; j < this->body->at(i)->size(); j++) {
				s += this->body->at(i)->at(j)->print();
				if(j+1 != this->body->at(i)->size())
					s += ", ";
			}
			if(i+1 != this->body->size())
					s += "\n\t|\t";
		}
		return s;
	}

	void Rule::valuate_rule(std::map<std::string, Value*> binding) {
		this->head->valuate_fact(binding);
		for(auto section: *this->body) {
			for(auto val: *section) {
				val->valuate_fact(binding);
			}
		}
	}
}
