#ifndef __SCANNER_H__
#define __SCANNER_H__

#if ! defined(yyFlexLexerOnce)
#include <FlexLexer.h>
#endif

#undef YY_DECL
#define YY_DECL int Inter4ql::Scanner::yylex()

#include "LineParser.tab.h"

namespace Inter4ql {

	/** @brief Scanner class, deriving from Flex generated class.
	 */
	class Scanner : public yyFlexLexer {
	public:
		/** @brief Default constructor
		 *
		 * @param arg_yyin  points the scanner input (default is standard input)
		 * @param arg_yyout points the scanner output (default is standard output)
		 */
		Scanner(std::istream* arg_yyin = 0, std::ostream* arg_yyout = 0) : yyFlexLexer(arg_yyin, arg_yyout) {
		}

		~Scanner() {
		}

		/** @brief
		 *
		 * @param lval lexer sematic output value
		 * @param lloc lexer location output value
		 * @return yylex() value from orginal flex generated lexer class
		 */
		int yylex(Inter4ql::LineParser::semantic_type * lval, Inter4ql::LineParser::location_type * lloc) {
			/* we are pointing arguments to class private members
			   to be able to get their values afterwards */
			yylval = lval;
			yylloc = lloc;

			return yylex();
		}

	private:
		/** @brief protected member of yyFlexLexer class
		 *
		 * @return type of next token in input
		 */
		int yylex();
		Inter4ql::LineParser::semantic_type * yylval;
		Inter4ql::LineParser::location_type * yylloc;
	};
}

#endif
