#ifndef __VALUATION_H__
#define __VALUATION_H__

#include "Logic.h"

#include <map>
#include <vector>

namespace Inter4ql {
	class Value;
	class Valuation {
	private:
		std::map<std::vector<Value *>, logic_type::type> valuation;
	public:
		/** @brief constructor
		 *
		 */
		Valuation();
		/** @brief adds element to valuation
		 *
		 * @param args values to add
		 * @param value logic value to assign to them
		 * @return true if element was successfully added
		 */
		bool add_element(const std::vector<Value *>& args, logic_type::type value);
		/** @brief gets logic value for a given arguments
		 *
		 * @return logic value
		 */
		logic_type::type get_logic_value(const std::vector<Value*>& vect);
		/** @brief removes element from valuation
		 *
		 * @param args values to remove
		 * @return true if element was successfully removed
		 */
		bool remove_element(const std::vector<Value *>& args);
		/** @brief expands current valuation by adding another variable
		 *
		 * @param vals possible values for new variable
		 */
		void expand_by(const std::vector<Value *>& vals);
		/** @brief remove all valuations that are inconsistent in argument
		 *
		 * @param vals valuation to check for inconsistencies
		 */
		void remove_inconsistent(const Valuation& val);
		/** @brief gets first logic value in the list (if no variables then it would be a global logic value)
		 *
		 * @return first logic value in the list
		 */
		logic_type::type get_first_logic_value() const;

		/** @brief checks whether all elements of valuation are unknown
		 *
		 * @return whether all element are unknown
		 */
		bool is_all_unknown() const;

		void print();

		decltype(valuation)::iterator begin() { return valuation.begin(); };
		const decltype(valuation)::const_iterator begin() const { return valuation.cbegin(); };
		decltype(valuation)::iterator end() { return valuation.end(); };
		const decltype(valuation)::const_iterator end() const { return valuation.cend(); };
		const decltype(valuation)::size_type size() const { return valuation.size(); };
		const decltype(valuation)::const_iterator find(const std::vector<Value *>& a) const {return this->valuation.find(a);};
	};
}

#endif
