#include "../Domain.h"
#include "../VariableSpace.h"
#include "../Functions.h"
#include "Bel.h"
#include <math.h>
#include <algorithm>
#include "../Exception.h"
#include "../Result.h"
#include <dirent.h>
#include <string.h>
#include <fstream>
#include <sstream>
#include <string>
#include <iostream>

namespace Inter4ql {
	Bel::Bel(VariableSpace *_vs, std::vector<ModuleGeneric *> *_modules) : ModuleInternal(_vs) {
		this->name = "BEL";
		this->modules = _modules;
		this->relations = new std::vector<Relation*>();
		this->disable_constraints = false;
	}

	Bel::~Bel() {}

	void Bel::reason()
	{

	}

	module_type::type Bel::get_module_type() const {
		return module_type::BEL;
	}

	Result ask_for_result_as(AsExpr* e, Fact* term, std::vector<ModuleGeneric *> *modules, bool bel_free_formula) {
		if (e->type_ == 0) {
			for(std::vector<ModuleGeneric *>::iterator it = modules->begin(); it != modules->end(); it++) {
				if ((*it)->get_name() == e->id_) {
					(*it)->set_disable_constraints(true);
					//printf("Asking %s for sth\n", (*it)->get_name().c_str());
					Result to_return;
					if (bel_free_formula) {
						to_return = (*it)->evaluate_expression_bf(term->get_bel_params());
					} else {
						to_return = (*it)->evaluate_expression(term->get_bel_params());
					}
					(*it)->set_disable_constraints(false);
					return to_return;
				}
			}
		} else {
			//printf("Asking shadowing module\n");
			Result right = ask_for_result_as(e->right_, term, modules, bel_free_formula);
			if (right.get_valuation().is_all_unknown()) {
				//printf("Nope, returned unknown... Asking left\n");
				return ask_for_result_as(e->left_, term, modules, bel_free_formula);
			} else {
				//printf("Yup, returning shadowed answer! %s\n", right.print(true, true).c_str());
				return right;
			}
		}
	}

	void ask_for_ids(AsExpr* e, std::vector<std::string> *ids) {
		if (e->type_ == 0) {
			ids->push_back(std::string(e->id_));
		} else {
			ask_for_ids(e->left_, ids);
			ask_for_ids(e->right_, ids);
		}
	}

	bool check_bel_constraints(AsExpr* e, std::vector<ModuleGeneric *> *modules, bool check_rigid, bool check_flexible) {
		if (e->type_ == 0) {
			for(std::vector<ModuleGeneric *>::iterator it = modules->begin(); it != modules->end(); it++) {
				if ((*it)->get_name() == e->id_) {
					return (*it)->check_constraints_local(check_rigid, check_flexible);
				}
			}
		} else {
			if (!check_bel_constraints(e->left_, modules, true, false))
				return false;
			if (!check_bel_constraints(e->right_, modules, true, check_flexible))
				return false;

			return true;
		}
	}

	Result Bel::ask_for_result(Fact * term) const
	{
		if(term->get_module() != "BEL") {
			return Result();
		}

		bool bel_free_query = term->is_bel_free_query();

		std::string belief_base = term->get_relation();
		if (belief_base.find("<") != -1) {
			int pos = 0;
			AsExpr* as_expr = parse_asExpr(belief_base, pos);

			if (!check_bel_constraints(as_expr, this->modules, true, true)) {
				delete as_expr;
				return Result();
			}

			Result toReturn = ask_for_result_as(as_expr, term, this->modules, bel_free_query);
			delete as_expr;
			return toReturn;
		}

		for(std::vector<ModuleGeneric *>::iterator it = this->modules->begin(); it != this->modules->end(); it++) {
			//printf("Checking module %s\n", (*it)->get_name().c_str());
			if ((*it)->get_name() == belief_base) {
				if (bel_free_query) {
					return (*it)->evaluate_expression_bf(term->get_bel_params());
				} else
					return (*it)->evaluate_expression(term->get_bel_params());
			}
		}

		Result ret;
		return ret;
	}

	std::vector<std::string> Bel::get_references(Fact * term) {
		std::vector<std::string> toReturn;
		std::string belief_base = term->get_relation();
		if (belief_base.find("<") != -1) {
			int pos = 0;
			AsExpr* as_expr = parse_asExpr(belief_base, pos);
			ask_for_ids(as_expr, &toReturn);
			delete as_expr;
		} else {
			toReturn.push_back(belief_base);
		}
		return toReturn;
	}

	bool Bel::constraints_violated() {
		return !check_constraints_local(true, true);
	}
}
