#include "../Domain.h"
#include "../VariableSpace.h"
#include "../Functions.h"
#include "../Exception.h"
#include "../Result.h"
#include "Plugin.h"
#include <dirent.h>
#include <string.h>
#include <fstream>
#include <sstream>
#include <string>
#include <iostream>

namespace Inter4ql {
    Plugin::Plugin(VariableSpace *_v) : ModuleExternal(_v) {
        this->name = "plugin";
        this->relations = new std::vector<Relation *>();

        DIR *pDIR;
        struct dirent *entry;
        if( pDIR=opendir("./plugins/") ){
                std::string name;
                while(entry = readdir(pDIR)){
                        name = "";

                        if( strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0 &&  strcmp(entry->d_name, "") != 0)
                                name.assign(entry->d_name, strlen(entry->d_name));

                        #if defined(WIN32) || defined(_WIN32) || defined(__WIN32) && !defined(__CYGWIN__)
                        std::string extension;
                        int dot = name.find(".") + 1;
                        if(dot == 0) dot = name.length();
                        extension = name.substr(dot);

                        int exitcode = 1;

                        if(extension == "exe")
                                exitcode = std::system((".\\plugins\\" + name + " interface >temp 2>NUL").c_str());
                                name = name.substr(0, name.find("."));
                        #else
                        int exitcode = std::system(("./plugins/" + name + " interface >temp 2>/dev/null").c_str());
                        #endif

                        if(exitcode == 0) {
                                std::ifstream myfile;
                                myfile.open ("temp");

                                std::string interface((std::istreambuf_iterator<char>(myfile)), std::istreambuf_iterator<char>());
                                //std::cout << "plugin '" << entry->d_name << "' zdeklarowal interface " << interface << std::endl;

                                myfile.close();

                                add_relation(name, interface);
                        }

                        std::remove("temp");
                }
                closedir(pDIR);
        }

    }

    Inter4ql::logic_type::type Plugin::runPlugin(std::string name, std::string params) {
        int exitcode;
        #if defined(WIN32) || defined(_WIN32) || defined(__WIN32) && !defined(__CYGWIN__)
                exitcode = std::system((".\\plugins\\" + name + ".exe calculate " + params + ">NUL 2>NUL").c_str());
        #else
                exitcode = std::system(("./plugins/" + name + " calculate " + params + ">/dev/null 2>/dev/null").c_str()) / 256;
        #endif

        //std::cout << "External call for " << name << " with params " << params << " ended with exitcode " << exitcode << std::endl;

        if(exitcode == 0) {
                return Inter4ql::logic_type::FALSE;
        } else {
                if(exitcode == 1) {
                        return Inter4ql::logic_type::TRUE;
                } else {
                        std::cout << "Plugin error exit code: " << exitcode << std::endl;
                        throw new Exception("Something went wrong while running plugin: " + name + "; with parameters: calculate " + params + "!  Make sure that this plugin exists in plugins directory!\n");
                }
        }
    }

	void Plugin::reason()
	{
	}

	Result Plugin::ask_for_result(Fact * term) const
	{
		if (term->get_module() != "plugin")
			return Result();
		std::string params;
		for (const auto& v : *term->get_params())
		{
			params += v->print(false) + " ";
		}
		auto return_val = runPlugin(term->get_relation(), params);
		Result ret;
		ret.add_element({}, return_val);
		return ret;
	}

    void Plugin::add_relation(std::string name, std::string interface) {

        std::vector<Domain *> *ii = new std::vector<Domain*> ();

        std::istringstream iss(interface);

        do
        {
                //TYPE = 1, VARIABLE = 2, ID = 3, STRING = 4, INTEGER = 5, REAL = 6, DATE = 7, DATE_TIME = 8, LOGIC = 9, INVALID = 10

                std::string sub;
                iss >> sub;

                if(sub == "LITERAL") {
                        ii->push_back(new Domain(variable_type::ID));
                        continue;
                }
                if(sub == "STRING") {
                        ii->push_back(new Domain(variable_type::STRING));
                        continue;
                }
                if(sub == "INTEGER") {
                        ii->push_back(new Domain(variable_type::INTEGER));
                        continue;
                }
                if(sub == "REAL") {
                        ii->push_back(new Domain(variable_type::REAL));
                        continue;
                }
                if(sub == "DATE") {
                        ii->push_back(new Domain(variable_type::DATE));
                        continue;
                }
                if(sub == "DATE_TIME") {
                        ii->push_back(new Domain(variable_type::DATE_TIME));
                        continue;
                }
                if(sub == "LOGIC") {
                        ii->push_back(new Domain(variable_type::LOGIC));
                        continue;
                }
        } while (iss);

        this->relations->push_back(new Relation(name, ii, "plugin"));
    }
}
